# -*- coding: utf-8 -*-
# the __all__ is generated
__all__ = []

# __init__.py structure:
# common code of the package
# export interface in __all__ which contains __all__ of its sub modules

# import all from submodule informer
from .informer import *
from .informer import __all__ as _informer_all
__all__ += _informer_all