# -*- coding: utf-8 -*-
# this file is generated by gen_kdata_schema function, dont't change it
from sqlalchemy.orm import declarative_base

from zvtm.contract.register import register_schema
from zvtm.domain.quotes import StockhkKdataCommon

KdataBase = declarative_base()


class Stockhk1dHfqKdata(KdataBase, StockhkKdataCommon):
    __tablename__ = 'stockhk_1d_hfq_kdata'


register_schema(providers=['em'], db_name='stockhk_1d_hfq_kdata', schema_base=KdataBase, entity_type='stockhk')

# the __all__ is generated
__all__ = ['Stockhk1dHfqKdata']