# the __all__ is generated
__all__ = []

# __init__.py structure:
# common code of the package
# export interface in __all__ which contains __all__ of its sub modules

# import all from submodule zen_factor
from .zen_factor import *
from .zen_factor import __all__ as _zen_factor_all
__all__ += _zen_factor_all

# import all from submodule domain
from .domain import *
from .domain import __all__ as _domain_all
__all__ += _domain_all