# -*- coding: utf-8 -*-
# the __all__ is generated
__all__ = []

# __init__.py structure:
# common code of the package
# export interface in __all__ which contains __all__ of its sub modules

# import all from submodule macd_factor
from .macd_factor import *
from .macd_factor import __all__ as _macd_factor_all
__all__ += _macd_factor_all